{

Copyright (c) 2014-2015, Fernando Biazi Nascimento
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of any organization.

}
unit FBNAboutDialog;

{$mode objfpc}{$H+}

interface

uses
  Classes, Controls, Dialogs, Forms, Graphics, LResources, SysUtils;

type

  { TFBNTextAboutItem }

  TFBNTextAboutItem = class(TCollectionItem)
  private
    { Private declarations }
    FCaption: ShortString;
    FText: TStrings;
    procedure SetText(AValue: TStrings);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(ACollection: TCollection); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Caption: ShortString read FCaption write FCaption;
    property Text: TStrings read FText write SetText;
  end;

  { TFBNTextAboutList }

  TFBNTextAboutList = class(TCollection)
  private
    function GetItems(Index: integer): TFBNTextAboutItem;
    procedure SetItems(Index: integer; AValue: TFBNTextAboutItem);
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create;
    function Add: TFBNTextAboutItem;
    function AddEx(Caption: ShortString; Text: TStrings): TFBNTextAboutItem;
    property Items[Index: integer]: TFBNTextAboutItem
      read GetItems write SetItems; default;
  published
    { Published declarations }
  end;


  { TFBNAboutDialog }

  TFBNAboutDialog = class(TCommonDialog)
  private
    FAboutForm: TForm;
    FCaptionBuild: ShortString;
    FCaptionCloseButton: ShortString;
    FCaptionCopyright: ShortString;
    FCaptionPName: ShortString;
    FCaptionPVersion: ShortString;
    FCaptionRelease: ShortString;
    FCaptionVersion: ShortString;
    FCaptionVersionTab: ShortString;
    FModalResult: TModalResult;
    FTextAboutList: TFBNTextAboutList;
    function FormCreated: boolean;
    procedure SetCaptionBuild(AValue: ShortString);
    procedure SetCaptionCloseButton(AValue: ShortString);
    procedure SetCaptionCopyright(AValue: ShortString);
    procedure SetCaptionPName(AValue: ShortString);
    procedure SetCaptionPVersion(AValue: ShortString);
    procedure SetCaptionRelease(AValue: ShortString);
    procedure SetCaptionVersion(AValue: ShortString);
    procedure SetCaptionVersionTab(AValue: ShortString);
    procedure SetTextAboutList(AValue: TFBNTextAboutList);
    { Private declarations }
  protected
    { Protected declarations }
    procedure AboutFormOnClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure AddEventsToForm;
    procedure FormatForm;
    procedure SetName(const NewName: TComponentName); override;
  public
    { Public declarations }
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
    property AboutForm: TForm read FAboutForm;
    procedure CreateAboutForm;
    function Execute: boolean; override;
    property ModalResult: TModalResult read FModalResult;
  published
    { Published declarations }
    property CaptionBuild: ShortString
      read FCaptionBuild write SetCaptionBuild;
    property CaptionCloseButton: ShortString
      read FCaptionCloseButton write SetCaptionCloseButton;
    property CaptionCopyright: ShortString
      read FCaptionCopyright write SetCaptionCopyright;
    property CaptionPName: ShortString
      read FCaptionPName write SetCaptionPName;
    property CaptionPVersion: ShortString
      read FCaptionPVersion write SetCaptionPVersion;
    property CaptionRelease: ShortString
      read FCaptionRelease write SetCaptionRelease;
    property CaptionVersion: ShortString
      read FCaptionVersion write SetCaptionVersion;
    property CaptionVersionTab: ShortString
      read FCaptionVersionTab write SetCaptionVersionTab;
    property Height default 250;
    property TextAboutList: TFBNTextAboutList
      read FTextAboutList write SetTextAboutList;
    property Width default 400;
  end;

procedure Register;

implementation

uses
  ComCtrls, ExtCtrls, FBNVersionInfo, resource, StdCtrls;

procedure Register;
begin
  {$I fbnaboutdialog_icon.lrs}
  RegisterComponents('FBN', [TFBNAboutDialog]);
  //RegisterPropertyEditor(PropertyType, PersistentClass, PropertyName
  //, EditorClass);
end;

{ TFBNAboutDialog }

function TFBNAboutDialog.FormCreated: boolean;
begin
  Result := AboutForm <> nil;
end;

procedure TFBNAboutDialog.SetCaptionBuild(AValue: ShortString);
begin
  if FCaptionBuild = AValue then
    Exit;
  FCaptionBuild := AValue;
end;

procedure TFBNAboutDialog.SetCaptionCloseButton(AValue: ShortString);
begin
  if FCaptionCloseButton = AValue then
    Exit;
  FCaptionCloseButton := AValue;
end;

procedure TFBNAboutDialog.SetCaptionCopyright(AValue: ShortString);
begin
  if FCaptionCopyright = AValue then
    Exit;
  FCaptionCopyright := AValue;
end;

procedure TFBNAboutDialog.SetCaptionPName(AValue: ShortString);
begin
  if FCaptionPName = AValue then
    Exit;
  FCaptionPName := AValue;
end;

procedure TFBNAboutDialog.SetCaptionPVersion(AValue: ShortString);
begin
  if FCaptionPVersion = AValue then
    Exit;
  FCaptionPVersion := AValue;
end;

procedure TFBNAboutDialog.SetCaptionRelease(AValue: ShortString);
begin
  if FCaptionRelease = AValue then
    Exit;
  FCaptionRelease := AValue;
end;

procedure TFBNAboutDialog.SetCaptionVersion(AValue: ShortString);
begin
  if FCaptionVersion = AValue then
    Exit;
  FCaptionVersion := AValue;
end;

procedure TFBNAboutDialog.SetCaptionVersionTab(AValue: ShortString);
begin
  if FCaptionVersionTab = AValue then
    Exit;
  FCaptionVersionTab := AValue;
end;

procedure TFBNAboutDialog.SetTextAboutList(AValue: TFBNTextAboutList);
begin
  if Assigned(AValue) then
    FTextAboutList.Assign(AValue);
end;

procedure TFBNAboutDialog.AddEventsToForm;
begin
  AboutForm.OnClose := @AboutFormOnClose;
end;

procedure TFBNAboutDialog.FormatForm;
begin
  if not FormCreated then
    exit;
  AboutForm.FormStyle := fsStayOnTop;
  AboutForm.Height := Height;
  AboutForm.Width := Width;
  if (csDesigning in ComponentState) then
    AboutForm.Position := poScreenCenter
  else
    AboutForm.Position := poOwnerFormCenter;
  AboutForm.BorderStyle := bsDialog;
end;

procedure TFBNAboutDialog.SetName(const NewName: TComponentName);
var
  UpdateTitle: boolean;
begin
  if Name = Title then
    UpdateTitle := True;
  inherited SetName(NewName);
  if UpdateTitle then
    Title := Name;
end;

procedure TFBNAboutDialog.AboutFormOnClose(Sender: TObject;
  var CloseAction: TCloseAction);
var
  CanClose: boolean;
begin
  CanClose := True;
  DoCanClose(CanClose);
  if CanClose then
    CloseAction := caHide
  else
    CloseAction := caNone;
end;

constructor TFBNAboutDialog.Create(TheOwner: TComponent);
var
  //strList: TStringList;
  tai: TFBNTextAboutItem;
begin
  inherited Create(TheOwner);
  FAboutForm := nil;
  FModalResult := mrNone;
  FTextAboutList := TFBNTextAboutList.Create;
  Height := 250;
  Width := 400;
  if (csDesigning in ComponentState) then
  begin
    tai := FTextAboutList.Add;
    tai.Caption := 'FBNControls';
    tai.Text.Add('This program uses one or more FBNControls components, that' +
      ' have the following license:');
    tai.Text.Add('');
    tai.Text.Add('Copyright (c) 2015, Fernando Biazi Nascimento');
    tai.Text.Add('All rights reserved.');
    tai.Text.Add('');
    tai.Text.Add('Redistribution and use in source and binary forms, with or' +
      ' without modification, are permitted provided that the following' +
      ' conditions are met:');
    tai.Text.Add('');
    tai.Text.Add('1. Redistributions of source code must retain the above' +
      ' copyright notice, this list of conditions and the following' +
      ' disclaimer.');
    tai.Text.Add('');
    tai.Text.Add('2. Redistributions in binary form must reproduce the above' +
      ' copyright notice, this list of conditions and the following' +
      ' disclaimer in the documentation and/or other materials provided with' +
      ' the distribution.');
    tai.Text.Add('');
    tai.Text.Add('THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND' +
      ' CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,' +
      ' INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF' +
      ' MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.' +
      ' IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR' +
      ' ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR' +
      ' CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF' +
      ' SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR' +
      ' BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF' +
      ' LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING' +
      ' NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS' +
      ' SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.');
  end;
end;

destructor TFBNAboutDialog.Destroy;
begin
  FTextAboutList.Free;
  inherited Destroy;
end;

procedure TFBNAboutDialog.CreateAboutForm;
var
  BtnPnl: TPanel;
  PgeCtl: TPageControl;
  TabSheets: TTabSheet;
  Labels: TLabel;
  Buttons: TButton;
  CollectionItem: TCollectionItem;
  Memos: TMemo;
  VI: TFBNVersionInfo;
  Stream: TResourceStream;
  t, l, tInc: integer;
begin
  if (csDesigning in ComponentState) then
  begin
    if (FAboutForm <> nil) then
      FAboutForm.Release;
    FAboutForm := TForm.Create(nil);
  end
  else
  begin
    if (FAboutForm <> nil) then
      exit;
    FAboutForm := TForm.Create(self.Owner);
  end;
  Stream := TResourceStream.CreateFromID(HINSTANCE, 1, PChar(RT_VERSION));
  try
    VI := TFBNVersionInfo.Create;
    VI.SetCustomRawDataStream(Stream);
    try
      FormatForm;
      AboutForm.Caption := Title;
      PgeCtl := TPageControl.Create(FAboutForm);
      PgeCtl.Align := alClient;
      TabSheets := PgeCtl.AddTabSheet;
      TabSheets.Caption := FCaptionVersionTab;
      { "Version" TabSheet contents }
      t := 10;
      l := 10;
      tInc := 20;
      if (FCaptionPName <> '') and (VI.ProductName <> '') then
      begin
        Labels := TLabel.Create(FAboutForm);
        Labels.Name := 'ProductName';
        Labels.Caption := FCaptionPName + ': ' + VI.ProductName;
        Labels.top := t;
        Labels.Left := l;
        Labels.Parent := TabSheets;
        t += tInc;
      end;
      if (FCaptionPVersion <> '') and (VI.ProductVersion <> '') then
      begin
        Labels := TLabel.Create(FAboutForm);
        Labels.Name := 'ProductVersion';
        Labels.Caption := FCaptionPVersion + ': ' + VI.ProductVersion;
        Labels.top := t;
        Labels.Left := l;
        Labels.Parent := TabSheets;
        t += tInc;
      end;
      if FCaptionVersion <> '' then
      begin
        Labels := TLabel.Create(FAboutForm);
        Labels.Name := 'LabelVersion';
        Labels.Caption := FCaptionVersion + ': ' +
          IntToStr(VI.MajorVersion) + '.' + IntToStr(VI.MinorVersion);
        Labels.top := t;
        Labels.Left := l;
        Labels.Parent := TabSheets;
        t += tInc;
      end;
      if FCaptionRelease <> '' then
      begin
        Labels := TLabel.Create(FAboutForm);
        Labels.Name := 'LabelRelease';
        Labels.Caption := FCaptionRelease + ': ' + IntToStr(VI.Release);
        Labels.top := t;
        Labels.Left := l;
        Labels.Parent := TabSheets;
        t += tInc;
      end;
      if FCaptionBuild <> '' then
      begin
        Labels := TLabel.Create(FAboutForm);
        Labels.Name := 'LabelBuild';
        Labels.Caption := FCaptionBuild + ': ' + IntToStr(VI.Build);
        Labels.top := t;
        Labels.Left := l;
        Labels.Parent := TabSheets;
        t += tInc;
      end;
      if (VI.LegalCopyright <> '') then
      begin
        Labels := TLabel.Create(FAboutForm);
        Labels.Name := 'LabelCopyright';
        if (FCaptionCopyright <> '') then
          Labels.Caption := FCaptionCopyright + ': ' + VI.LegalCopyright
        else
          Labels.Caption := VI.LegalCopyright;
        Labels.top := t;
        Labels.Left := l;
        Labels.Parent := TabSheets;
        t += tInc;
      end;
      { Buttons }
      BtnPnl := TPanel.Create(FAboutForm);
      BtnPnl.Height := 30;
      BtnPnl.BevelOuter := bvNone;
      BtnPnl.Align := alBottom;
      Buttons := TButton.Create(FAboutForm);
      Buttons.Caption := FCaptionCloseButton;
      Buttons.Anchors := [];
      Buttons.left := (BtnPnl.Width - Buttons.Width) div 2;
      Buttons.top := (BtnPnl.Height - Buttons.Height) div 2;
      Buttons.Parent := BtnPnl;
      Buttons.ModalResult := mrClose;
      { Put the controls on the form }
      PgeCtl.Parent := FAboutForm;
      BtnPnl.Parent := FAboutForm;
      FormatForm;
      AddEventsToForm;
    finally
      VI.Free;
    end;
  finally
    Stream.Free;
  end;

  { User defined TabSheets }
  for CollectionItem in TextAboutList do
  begin
    TabSheets := PgeCtl.AddTabSheet;
    TabSheets.Caption := TFBNTextAboutItem(CollectionItem).Caption;
    Memos := TMemo.Create(FAboutForm);
    Memos.Align := alClient;
    Memos.ScrollBars := ssAutoVertical;
    Memos.Parent := TabSheets;
    Memos.Lines.Assign(TFBNTextAboutItem(CollectionItem).Text);
  end;

end;

function TFBNAboutDialog.Execute: boolean;
begin
  CreateAboutForm;
  if csDesigning in ComponentState then
  begin
    FAboutForm.ShowModal;
    FAboutForm.Release;
    FAboutForm := nil;
    Exit;
  end;
  FModalResult := FAboutForm.ShowModal;
  Result := not (FModalResult in [mrCancel, mrAbort, mrNo]);
end;

{ TFBNTextAboutItem }

procedure TFBNTextAboutItem.SetText(AValue: TStrings);
begin
  if (AValue <> nil) then
    FText.Assign(AValue);
end;

constructor TFBNTextAboutItem.Create(ACollection: TCollection);
begin
  if Assigned(ACollection) and (ACollection is TFBNTextAboutList) then
    inherited Create(ACollection);
  FCaption := '';
  FText := TStringList.Create;
end;

destructor TFBNTextAboutItem.Destroy;
begin
  FText.Free;
  inherited Destroy;
end;

{ TFBNTextAboutList }

function TFBNTextAboutList.GetItems(Index: integer): TFBNTextAboutItem;
begin
  Result := TFBNTextAboutItem(inherited Items[Index]);
end;

procedure TFBNTextAboutList.SetItems(Index: integer;
  AValue: TFBNTextAboutItem);
begin
  Items[Index].Assign(AValue);
end;

constructor TFBNTextAboutList.Create;
begin
  inherited Create(TFBNTextAboutItem);
end;

function TFBNTextAboutList.Add: TFBNTextAboutItem;
begin
  Result := inherited Add as TFBNTextAboutItem;
end;

function TFBNTextAboutList.AddEx(Caption: ShortString;
  Text: TStrings): TFBNTextAboutItem;
begin
  Result := inherited Add as TFBNTextAboutItem;
  Result.Caption := Caption;
  Result.Text := Text;
end;

end.
