{

Copyright (c) 2014-2015, Fernando Biazi Nascimento
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of any organization.
}
unit FBNBufDataset;

{$mode objfpc}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics,
  Dialogs, BufDataset, DB;

type
  TFBNBufDataset = class(TBufDataset)
  const
    Version = 1;
  private
    { Private declarations }
    FChunkSize: longword;
  protected
    { Protected declarations }
    procedure InternalInitFieldDefs; override;
    procedure InternalOpen; override;
  public
    { Public declarations }
    procedure AppendToStream(AStream: TStream);
    // ; Format: TDataPacketFormat = dfBinary
    procedure ClearData;
    procedure LoadFromStreamAt(AStream: TStream);
  published
    { Published declarations }
    property ChunkSize: longword
      read FChunkSize write FChunkSize default 524288;
  end;

procedure Register;

implementation

uses
  FMTBcd;

{ TFBNBufDataset }

procedure TFBNBufDataset.AppendToStream(AStream: TStream);
// ; Format: TDataPacketFormat = dfBinary
var
  ctrlCode: byte;
  strBuf: ansistring;
  strWBuf: WideString;
  chunk: TMemoryStream; // chunks of data to write on one operation
  bm: TBookmark;
  i: integer;

  procedure StreamWriteAnsiStrChunk(S: string; chunkSize: cardinal;
    Buf: TMemoryStream; AStream: TStream);
  begin
    if chunkSize <> 0 then
    begin
      Buf.WriteAnsiString(S);
      if Buf.Size > chunkSize then
      begin
        Buf.Position := 0;
        AStream.CopyFrom(Buf, Buf.Size);
        Buf.Position := 0;
        Buf.Size := 0;
      end;
    end
    else
      AStream.WriteAnsiString(S);
  end;

  function StreamWriteWithChunk(const Buffer; Count: cardinal;
    chunkSize: cardinal; Buf: TMemoryStream; AStream: TStream): longint;
  begin
    if chunkSize <> 0 then
    begin
      Result := Buf.Write(Buffer, Count);
      if Buf.Size > chunkSize then
      begin
        Buf.Position := 0;
        AStream.CopyFrom(Buf, Buf.Size);
        Buf.Position := 0;
        Buf.Size := 0;
      end;
    end
    else
      Result := AStream.Write(Buffer, Count);
  end;

  procedure StreamWriteWordWithChunk(w: word; chunkSize: cardinal;
    Buf: TMemoryStream; AStream: TStream);
  begin
    if chunkSize <> 0 then
    begin
      Buf.WriteWord(w);
      if Buf.Size > chunkSize then
      begin
        Buf.Position := 0;
        AStream.CopyFrom(Buf, Buf.Size);
        Buf.Position := 0;
        Buf.Size := 0;
      end;
    end
    else
      AStream.WriteWord(w);
  end;

begin
  CheckBiDirectional;
  bm := GetBookmark;
  try
    if FChunkSize <> 0 then
      chunk := TMemoryStream.Create;

    try
      { Write Version information }
      StreamWriteWordWithChunk(Version, FChunkSize, chunk, AStream);

      { Write Header to make possible some check on opening }
      ctrlCode := 1; // ACSII 001: Start of Header
      StreamWriteWithChunk(ctrlCode, 1, FChunkSize, chunk, AStream);
      for i := 0 to FieldCount - 1 do
        if not (Fields[i].FieldKind in [fkCalculated, fkLookup]) then
        begin
          ctrlCode := 31; // ACSII 031: Unit Separator
          StreamWriteWithChunk(ctrlCode, 1, FChunkSize, chunk, AStream);
          StreamWriteAnsiStrChunk(Fields[i].FieldName, FChunkSize,
            chunk, AStream);
        end;
      ctrlCode := 2; // ACSII 002: Start of text
      StreamWriteWithChunk(ctrlCode, 1, FChunkSize, chunk, AStream);

      { Write each record on the destination stream }
      First;
      while not EOF do
      begin
        ctrlCode := 30; // ACSII 030: Record Separator
        StreamWriteWithChunk(ctrlCode, 1, FChunkSize, chunk, AStream);
        for i := 0 to FieldCount - 1 do
          if not (Fields[i].FieldKind in [fkCalculated, fkLookup]) then
          begin
            ctrlCode := 31; // ACSII 031: Unit Separator
            StreamWriteWithChunk(ctrlCode, 1, FChunkSize, chunk, AStream);

            if not Fields[i].IsBlob then
            begin
              StreamWriteWordWithChunk(Fields[i].DataSize,
                FChunkSize, chunk, AStream);
              case Fields[i].DataType of
                ftString:
                begin
                  strBuf := Fields[i].AsString;
                  StreamWriteWithChunk(strBuf[1], Fields[i].DataSize,
                    FChunkSize, chunk, AStream);
                end;
                ftSmallint, ftInteger, ftWord: StreamWriteWithChunk(
                    Fields[i].AsInteger, Fields[i].DataSize,
                    FChunkSize, chunk, AStream);
                ftBoolean: StreamWriteWithChunk(Fields[i].AsBoolean,
                    Fields[i].DataSize, FChunkSize, chunk, AStream);
                { TODO : there is some problem with these data-type }
                (* ftFloat: StreamWriteWithChunk(Fields[i].AsFloat,
                    Fields[i].DataSize, FChunkSize, chunk, AStream);
                ftCurrency: StreamWriteWithChunk(Fields[i].AsCurrency,
                    Fields[i].DataSize, FChunkSize, chunk, AStream); *)
                ftBCD: StreamWriteWithChunk(Fields[i].AsBCD,
                    Fields[i].DataSize,
                    FChunkSize, chunk, AStream);
                { TODO : there is some problem with these data-type }
                (* ftDate, ftTime, ftDateTime: StreamWriteWithChunk(
                    Fields[i].AsDateTime, Fields[i].DataSize, FChunkSize,
                    chunk, AStream); *)
                ftBytes, ftVarBytes:
                begin
                  strBuf := Fields[i].AsString;
                  StreamWriteWithChunk(strBuf[1], Fields[i].DataSize,
                    FChunkSize, chunk, AStream);
                end;
                ftAutoInc: StreamWriteWithChunk(Fields[i].AsInteger,
                    Fields[i].DataSize, FChunkSize, chunk, AStream);
                ftBlob, ftGraphic, ftFmtMemo, ftParadoxOle, ftDBaseOle,
                ftTypedBinary, ftCursor, ftFixedChar:
                begin
                  strBuf := Fields[i].AsString;
                  StreamWriteWithChunk(strBuf[1], Fields[i].DataSize,
                    FChunkSize, chunk, AStream);
                end;
                ftWideString: StreamWriteWithChunk(Fields[i].AsWideString,
                    Fields[i].DataSize, FChunkSize, chunk, AStream);
                ftLargeint: StreamWriteWithChunk(Fields[i].AsLargeInt,
                    Fields[i].DataSize, FChunkSize, chunk, AStream);
                ftADT, ftArray, ftReference, ftDataSet, ftOraBlob,
                ftOraClob, ftVariant, ftInterface, ftIDispatch,
                ftGuid:
                begin
                  strBuf := Fields[i].AsString;
                  StreamWriteWithChunk(strBuf[1], Fields[i].DataSize,
                    FChunkSize, chunk, AStream);
                end;
                { TODO : there is some problem with this data-type }
                (* ftTimeStamp: StreamWriteWithChunk(Fields[i].AsDateTime,
                    Fields[i].DataSize, FChunkSize, chunk, AStream); *)
                ftFMTBcd: StreamWriteWithChunk(Fields[i].AsInteger,
                    Fields[i].DataSize, FChunkSize, chunk, AStream);
                ftFixedWideChar, ftWideMemo:
                begin
                  strWBuf := Fields[i].AsWideString;
                  StreamWriteWithChunk(strWBuf, Fields[i].DataSize,
                    FChunkSize, chunk, AStream);
                end;
                else
                begin
                  strBuf := Fields[i].AsString;
                  StreamWriteWithChunk(strBuf[1], Fields[i].DataSize,
                    FChunkSize, chunk, AStream);
                end;
              end;
            end
            else
            begin
              StreamWriteWordWithChunk(
                (Fields[i] as TBlobField).BlobSize, FChunkSize,
                chunk, AStream);
              strBuf := Fields[i].AsString;
              StreamWriteWithChunk(strBuf[1],
                (Fields[i] as TBlobField).BlobSize,
                FChunkSize, chunk, AStream);
            end;
          end;
        Next;
      end;

      ctrlCode := 3; // ACSII 003: End of text
      StreamWriteWithChunk(ctrlCode, 1, FChunkSize, chunk, AStream);

      if FChunkSize <> 0 then
      begin
        if chunk.Size > 0 then
          AStream.CopyFrom(chunk, chunk.Size);
        chunk.Position := 0;
        chunk.Size := 0;
      end;
    finally
      if FChunkSize <> 0 then
        chunk.Free;
    end;

    if bm <> nil then
      if BookmarkValid(bm) then
        GotoBookmark(bm);
  finally
    FreeBookmark(bm);
  end;
end;

procedure TFBNBufDataset.ClearData;
begin
  First;
  while not EOF do
    Delete;
end;

procedure TFBNBufDataset.InternalInitFieldDefs;
begin

end;

procedure TFBNBufDataset.InternalOpen;
begin
  if DefaultFields then
    CreateFields;
  BindFields(True);
  inherited;
end;

procedure TFBNBufDataset.LoadFromStreamAt(AStream: TStream);
var
  ctrlCode: byte;
  ValueSize: word;
  bolBuf: boolean;
  bdcBuf: tBCD;
  intBuf: longint;
  lintBuf: LargeInt;
  strBuf: ansistring;
  strWBuf: WideString;
  i: integer;
  roFields: array of boolean;

  procedure raiseEVersion;
  begin
    raise Exception.Create('Incompatible file version');
  end;

  procedure raiseEFormat;
  begin
    raise EFormatError.Create('Invalid Format');
  end;

begin
  CheckBiDirectional;
  ClearData;

  { Store read only property of each field and make them read/write }
  SetLength(roFields, FieldCount);
  for i := 0 to FieldCount - 1 do
  begin
    roFields[i] := Fields[i].ReadOnly;
    Fields[i].ReadOnly := False;
  end;

  try
    { Read Version information }
    if AStream.ReadWord <> Version then
      raiseEVersion;

    { Read Header and compare to each column name }
    ctrlCode := AStream.ReadByte;
    if ctrlCode <> 1 then // ACSII 001: Start of Header
      raiseEFormat;

    for i := 0 to FieldCount - 1 do
      if not (Fields[i].FieldKind in [fkCalculated, fkLookup]) then
      begin
        ctrlCode := AStream.ReadByte;
        if ctrlCode <> 31 then // ACSII 031: Unit Separator
          raiseEFormat;
        if AStream.ReadAnsiString <> Fields[i].FieldName then
          raiseEFormat;
      end;

    ctrlCode := AStream.ReadByte;
    if ctrlCode <> 2 then // ACSII 002: Start of text
      raiseEFormat;

    ctrlCode := AStream.ReadByte;
    if not (ctrlCode in [3, 30]) then
      // ACSII 030: Record Separator; 003: End of text
      raiseEFormat;

    { Read each record  up to the EOT char }
    while ctrlCode = 30 do // ACSII 030: Record Separator
    begin
      Append;
      try
        for i := 0 to FieldCount - 1 do
          if not (Fields[i].FieldKind in [fkCalculated, fkLookup]) then
          begin
            ctrlCode := AStream.ReadByte;
            if ctrlCode <> 31 then // ACSII 031: Unit Separator
              raiseEFormat;

            if not Fields[i].IsBlob then
            begin
              ValueSize := AStream.ReadWord;
              case Fields[i].DataType of
                ftString:
                begin
                  SetLength(strBuf, ValueSize);
                  AStream.ReadBuffer(strBuf[1], ValueSize);
                  Fields[i].AsString := strBuf;
                end;
                ftSmallint, ftInteger, ftWord:
                begin
                  intBuf := 0;
                  AStream.ReadBuffer(intBuf, Fields[i].DataSize);
                  Fields[i].AsInteger := intBuf;
                end;
                ftBoolean:
                begin
                  bolBuf := False;
                  AStream.ReadBuffer(bolBuf, Fields[i].DataSize);
                  Fields[i].AsBoolean := bolBuf;
                end;
                { TODO : there is some problem with these data-type }
            (* ftFloat: AStream.ReadBuffer(Fields[i].AsFloat,
                Fields[i].DataSize);
            ftCurrency: AStream.ReadBuffer(Fields[i].AsCurrency,
                Fields[i].DataSize); *)
                ftBCD:
                begin
                  bdcBuf := 0;
                  AStream.ReadBuffer(bdcBuf, Fields[i].DataSize);
                  Fields[i].AsBCD := bdcBuf;
                end;
                { TODO : there is some problem with these data-type }
            (* ftDate, ftTime, ftDateTime: AStream.ReadBuffer(
                Fields[i].AsDateTime, Fields[i].DataSize); *)
                ftBytes, ftVarBytes:
                begin
                  SetLength(strBuf, ValueSize);
                  AStream.ReadBuffer(strBuf[1], ValueSize);
                  Fields[i].AsString := strBuf;
                end;
                ftAutoInc:
                begin
                  intBuf := 0;
                  AStream.ReadBuffer(intBuf, Fields[i].DataSize);
                  { TODO : This ReadOnly stuf should not be needed!
The code on start and end of the method shoud be enough. }
                  Fields[i].ReadOnly := False;
                  Fields[i].AsInteger := intBuf;
                  Fields[i].ReadOnly := True;
                end;
                ftBlob, ftGraphic, ftFmtMemo, ftParadoxOle, ftDBaseOle,
                ftTypedBinary, ftCursor, ftFixedChar:
                begin
                  SetLength(strBuf, ValueSize);
                  AStream.ReadBuffer(strBuf[1], ValueSize);
                  Fields[i].AsString := strBuf;
                end;
                ftWideString:
                begin
                  SetLength(strWBuf, Fields[i].DataSize);
                  AStream.ReadBuffer(strWBuf, Fields[i].DataSize);
                  Fields[i].AsWideString := strWBuf;
                end;
                ftLargeint:
                begin
                  lintBuf := 0;
                  AStream.ReadBuffer(lintBuf, Fields[i].DataSize);
                  Fields[i].AsLargeInt := lintBuf;
                end;
                ftADT, ftArray, ftReference, ftDataSet, ftOraBlob,
                ftOraClob, ftVariant, ftInterface, ftIDispatch,
                ftGuid:
                begin
                  SetLength(strBuf, ValueSize);
                  AStream.ReadBuffer(strBuf[1], ValueSize);
                  Fields[i].AsString := strBuf;
                end;
                { TODO : there is some problem with this data-type }
            (* ftTimeStamp: AStream.ReadBuffer(Fields[i].AsDateTime,
                Fields[i].DataSize); *)
                ftFMTBcd:
                begin
                  intBuf := 0;
                  AStream.ReadBuffer(intBuf, Fields[i].DataSize);
                  Fields[i].AsInteger := intBuf;
                end;
                ftFixedWideChar, ftWideMemo:
                begin
                  SetLength(strBuf, ValueSize);
                  AStream.ReadBuffer(strBuf[1], ValueSize);
                  Fields[i].AsString := strBuf;
                end;
                else
                begin
                  SetLength(strBuf, ValueSize);
                  AStream.ReadBuffer(strBuf[1], ValueSize);
                  Fields[i].AsString := strBuf;
                end;
              end;
            end
            else
            begin
              ValueSize := AStream.ReadWord;
              SetLength(strBuf, ValueSize);
              AStream.ReadBuffer(strBuf[1], ValueSize);
              Fields[i].AsString := strBuf;
            end;
          end;
        Post;
      except
        Cancel; // Cancel new record
        raise; // ropagate exception so that it can be handled
      end;
      ctrlCode := AStream.ReadByte;
    end;
    if ctrlCode <> 3 then // ACSII 003: End of text
      raiseEFormat;

  finally
    { Restore read only property of each field }
    for i := 0 to FieldCount - 1 do
      Fields[i].ReadOnly := roFields[i];
  end;
end;

procedure Register;
begin
  {$I fbnbufdataset_icon.lrs}
  RegisterComponents('FBN', [TFBNBufDataset]);
end;

end.
